import uuid

from django.conf import settings
from django.contrib.contenttypes.fields import GenericRelation
from django.core.validators import MinLengthValidator
from django.db import models


class Budget(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=100, validators=[MinLengthValidator(1)])
    total = models.DecimalField(max_digits=12, decimal_places=3, default=0)
    memberships = GenericRelation("users.Right", "content_id")

    def __str__(self) -> str:
        return self.name


class BudgetEntry(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created = models.DateTimeField(auto_now_add=True)

    budget = models.ForeignKey(Budget, on_delete=models.CASCADE, related_name="entries")
    description = models.CharField(max_length=255, validators=[MinLengthValidator(1)])
    value = models.DecimalField(max_digits=12, decimal_places=3)

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="budget_entries",
    )
    process = models.ForeignKey(
        "process.Process",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="budget_entries",
    )

    class Meta:
        verbose_name_plural = "budget entries"

    def __str__(self) -> str:
        return self.description
